/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OtterXMLCNode.java,v 1.5 2000/06/02 08:32:33 pmahar Exp $
 */
package org.enhydra.kelp.common.node;

// Kelp imports
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;

// Enhydra imports
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.commands.xmlc.XMLCOptions;
import org.enhydra.xml.xmlc.commands.options.OptionsParser;
import org.enhydra.xml.xmlc.commands.options.OptionSet;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.5 $
 */
public class OtterXMLCNode implements OtterResourceNode {
    public final static int   CLASS_NAME_DEFAULT = 1;
    public final static int   CLASS_NAME_CUSTOM = 2;
    public final static int   CLASS_NAME_MAPPED = 3;
    private final String      CUSTOM_CLASS_NAME =
        "enhydra.xmlc.customClassName";
    private final String      CLASS_NAME_TYPE = "enhydra.xmlc.classNameType";
    private Options           options;
    private OtterResourceNode resourceNode;

    /**
     * Create a node to use when calling XMLC.
     *
     */
    public OtterXMLCNode(OtterResourceNode r) {
        resourceNode = r;
        initOptions();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCOptionFilename() {
        return resourceNode.getXMLCOptionFilename();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setXMLCOptionFilename(String n) {
        resourceNode.setXMLCOptionFilename(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCParameters() {
        return resourceNode.getXMLCParameters();
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setXMLCParameters(String p) {
        resourceNode.setXMLCParameters(p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSelected() {
        return resourceNode.isSelected();
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setSelected(boolean b) {
        resourceNode.setSelected(b);
        if (b) {
            setStatic(false);
        }
    }


    public boolean isStatic() {
        return resourceNode.isStatic();
    }

    public void setStatic(boolean b) {
        resourceNode.setStatic(b);
        if (b) {
            setSelected(false);
        }
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
        return resourceNode.getFileSeparatorChar();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilePath() {
        return resourceNode.getFilePath();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getCustomClassName() {
        String custom = getProperty(CUSTOM_CLASS_NAME);

        if (custom == null) {
            custom = new String();
        }
        return custom;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setCustomClassName(String n) {
        setProperty(CUSTOM_CLASS_NAME, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
        return resourceNode.getNativeNode();
    }

    /**
     * Method declaration
     *
     *
     * @param o
     *
     * @see
     */
    public void setNativeNode(Object o) {
        resourceNode.setNativeNode(o);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return resourceNode.getProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @return
     *
     * @see
     */
    public String getProperty(String n) {
        return resourceNode.getProperty(n);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     *
     * @see
     */
    public void setProperty(String n, String v) {
        resourceNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     *
     * @see
     */
    public void setProperty(String n, int v) {
        resourceNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getClassNameType() {

        // When getting the project scope overrides
        // local type.
        int    type;
        int    projectScope;
        String stringType = getProperty(CLASS_NAME_TYPE);

        projectScope = getProject().getMapScope();
        type = PropUtil.stringToInt(stringType, CLASS_NAME_MAPPED);
        switch (type) {
        case CLASS_NAME_DEFAULT:
        case CLASS_NAME_CUSTOM:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                type = CLASS_NAME_MAPPED;
                setProperty(CLASS_NAME_TYPE, type);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                type = CLASS_NAME_DEFAULT;
                setProperty(CLASS_NAME_TYPE, type);
            }
            break;
        }
        return type;
    }

    /**
     * Method declaration
     *
     *
     * @param newType
     *
     * @see
     */
    public void setClassNameType(int newType) {

        // If conflict exists with project scope,
        // change project scope.
        int oldType = getClassNameType();
        int projectScope = getProject().getMapScope();

        setProperty(CLASS_NAME_TYPE, newType);
        syncProjectToType(newType);
        if ((newType != oldType) && (newType != CLASS_NAME_CUSTOM)) {
            initClassName();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param forPrint
     *
     * @return
     *
     * @see
     */
    private String[] getAllParameters(boolean forPrint) {
        StringBuffer    buf = new StringBuffer();
        String[]        parameters = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        if (resourceNode.getProject().getXMLCParameters() != null) {
            buf.append(resourceNode.getProject().getXMLCParameters().trim());
        }
        if (forPrint) {
            buf.append("#");
        } else {
            buf.append(" ");
        }
        if (getXMLCParameters() != null) {
            buf.append(getXMLCParameters().trim());
        }
        if (forPrint) {
            tokenizer = new StringTokenizer(buf.toString().trim(), "#");
        } else {
            tokenizer = new StringTokenizer(buf.toString().trim());
        }
        count = tokenizer.countTokens();
        parameters = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            parameters[index] = tokenizer.nextToken();
            index++;
        }
        return parameters;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String[] getAllOptionFilenames() {
        StringBuffer    buf = new StringBuffer();
        File            file = null;
        String[]        filenames = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        if ((resourceNode.getProject().getXMLCOptionFilename() != null)
                && (resourceNode.getProject().getXMLCOptionFilename().trim().length()
                    > 0)) {

            // must insert project first so it can be overriden
            file =
                new File(resourceNode.getProject().getXMLCOptionFilename());
            if (file.exists() && file.isFile()
                    && file.getName().toLowerCase().endsWith(".xmlc")) {
                buf.append(file.getAbsolutePath());
            }
        }
        buf.append("#");
        if ((getXMLCOptionFilename() != null)
                && getXMLCOptionFilename().trim().length() > 0) {
            file = new File(getXMLCOptionFilename());
            if (file.exists() && file.isFile()
                    && file.getName().toLowerCase().endsWith(".xmlc")) {
                buf.append(file.getAbsolutePath());
            }
        }
        tokenizer = new StringTokenizer(buf.toString().trim(), "#");
        count = tokenizer.countTokens();
        filenames = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            filenames[index] = tokenizer.nextToken();
            index++;
        }
        return filenames;
    }

    /**
     * Method declaration
     *
     *
     * @param type
     *
     * @see
     */
    private void syncProjectToType(int type) {
        int projectScope = getProject().getMapScope();

        switch (type) {
        case CLASS_NAME_CUSTOM:
        case CLASS_NAME_DEFAULT:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        }
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
        if (e == null) {
            resourceNode.setException(null);
        } else {
            resourceNode.setException(new LocalException(e));
        }
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
        return resourceNode.getException();
    }

    /**
     * Get the XMLC options selected this HTML/WML file.
     */
    public Options getOptions() {
        return options;
    }

    /**
     * Add the generated Java source file to the IDE project. Do not call
     * this when using Win32 JBuilder as it can't handle the width of source
     * lines generated by XMLC.
     */
    public void replaceGeneratedSource(boolean forRecomp) {
        if ((options.getJavaClassSource().exists())
                && (getProject().isOpenBuild())) {
            int count = 1;
            if (forRecomp) {
              count = 2;
            }
            String[] source = new String[count];
            OtterNodeFactory factory = null;
            source[0] = options.getJavaClassSource().getAbsolutePath();
            factory = getProject().getNodeFactory();

            if (forRecomp) {
              source[1] = options.getJavaInterfaceSource().getAbsolutePath();
            }
            factory.replaceGeneratedSource(getProject(), resourceNode,
                                           source);

        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void initProjectOptions() {
        options.setKeep(true);         // always keep Java source
        options.setNoCompile(true);    // never compile Java source

        // loging options
        options.setParseInfo(getProject().isParseInfoSelected());
        options.setDump(getProject().isDumpSelected());
        options.setVerbose(getProject().isVerboseSelected());
        options.setPrintAccessMethods(getProject().isPrintAccessMethodsSelected());

        // Call XMLC with the do not call javac option.
    }

    /**
     * Initialize the XMLC options.
     *
     * @throws XMLCException
     * Thrown when options are invalid.
     *
     */
    private void initOptions() {
        options = new Options();
        initClassName();
        initProjectOptions();
        File sourceDoc = null;

        sourceDoc = new File(getFilePath());
        options.setSourceDocument(sourceDoc, true);
        initJavaSourceFile();
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void readParameters() throws XMLCException, IOException {
        String[] parameters = getAllParameters(false);

        if ((parameters.length > 0)) {
            OptionsParser op = getNewOptionsParser(options);

            callParse(op, options, parameters);
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void readOptionFiles() throws XMLCException, IOException {
        String[] files = getAllOptionFilenames();

        if ((files.length > 0)) {
            OptionsParser op = getNewOptionsParser(options);

            callParse(op, options, files);
        }
    }

    // This method allows us to create a new instance using the
    // 2.3 or 3.0 XMLC API.

    /**
     * Method declaration
     *
     *
     * @param options
     *
     * @return
     *
     * @see
     */
    private OptionsParser getNewOptionsParser(Options options) {
        OptionsParser parser = null;

        try {
            Class         opClass =
                Class.forName("org.enhydra.xml.xmlc.commands.options.OptionsParser");
            Constructor[] opCon = opClass.getConstructors();
            Class[]       params = opCon[0].getParameterTypes();
            Object[]      values = new Object[0];

            if (params.length == 1) {
                values = new Object[1];
                values[0] = options;
            }
            parser = (OptionsParser) opCon[0].newInstance(values);
        } catch (Exception e) {
            parser = null;
            e.printStackTrace();
        }
        return parser;
    }

    /**
     * Method declaration
     *
     *
     * @param parser
     * @param options
     * @param args
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void callParse(OptionsParser parser, Options options,
                           String[] args) throws IOException, XMLCException {
        Class    opClass = parser.getClass();
        Class[]  params = new Class[1];
        Object[] values = null;
        Method   method = null;

        params[0] = args.getClass();
        try {
            try {
                method = opClass.getMethod("parse", params);
                values = new Object[1];
                values[0] = args;
            } catch (NoSuchMethodException e) {
                params = new Class[2];
                params[0] =
                    Class.forName("org.enhydra.xml.xmlc.commands.options.OptionSet");
                params[1] = args.getClass();
                method = opClass.getMethod("parse", params);
                values = new Object[2];
                values[0] = options;
                values[1] = args;
            }
            method.invoke(parser, values);
        } catch (InvocationTargetException e) {
            if (e.getTargetException() instanceof IOException) {
                throw (IOException) e.getTargetException();
            } else if (e.getTargetException() instanceof XMLCException) {
                throw (XMLCException) e.getTargetException();
            } else {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        // op.parse(args); // 3.0
        // op.parse(options, args); //2.3
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initJavaSourceFile() {
        StringBuffer filename = new StringBuffer();
        int          index = -1;

        filename.append(getGenToDirectory().toString());
        filename.append("" + getProject().getFileSeparatorChar());
        String className = options.getClassName();
        if (options.getPackageName() != null) {
            index = className.indexOf(options.getPackageName());
        }
        if (index > -1) {
            className = className.substring(options.getPackageName().length()
                                            + 1);
        }
        index = className.lastIndexOf(".");
        if (index > -1) {
            className = className.substring(index + 1);
        }
        filename.append(className);
        filename.append(".java");
        File f = new File(filename.toString());
        options.setJavaClassSource(f);
    }

    /**
     * Get the generate to directory and create the directory if needed.
     * This is where the Java source files are created.
     */
    private File getGenToDirectory() {
        String genTo = getGenToPath();
        File   genDir = new File(genTo);

        if (genDir.exists()) {
            if (!genDir.isDirectory()) {
                System.err.println("Unable to write to directory: " + genTo);
            }
        } else {
            genDir.mkdirs();
        }
        return genDir;
    }

    /**
     * Get the generate to directory as a string. This is where XMLC will create
     * Java source files.
     */
    private String getGenToPath() {
        String       projectGenTo = getProject().getGenerateToDirectory();
        String       packagePath = getPackagePath();
        StringBuffer buf = new StringBuffer();

        if (projectGenTo == null) {
            buf.append(resourceNode.getProject().getSourcePathOf(resourceNode));
        } else {
            buf.append(projectGenTo);
        }
        if (packagePath.length() > 0) {
            buf.append(File.separator);
            buf.append(packagePath);
        }
        return buf.toString();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String getPackagePath() {
        String packPath = new String();

        if (options.getPackageName() != null) {
            packPath = options.getPackageName();
        } else if (options.getClassName() != null) {
            int index = options.getClassName().lastIndexOf('.');

            if (index > -1) {
                packPath = options.getClassName().substring(0, index);
            }
        }
        packPath = packPath.replace('.', File.separatorChar);
        return packPath;
    }

    /**
     * Create a class name for the given HTML/WML file and map the
     * package name as needed. Mapping allows users to  specify
     * a package name to use for all the HTML/WML files in a directory.
     */
    private void initClassName() {
        Mapper mapper = new Mapper(getProject());
        String className = mapper.getClassName(this);
        options.setClassName(className);
    }

    private String baseOut() {
        StringBuffer buf = new StringBuffer();
        String path = new String();
        buf.append(getProject().getOutputDirectory());
        buf.append(File.separator);
        path = options.getClassName();
        path = path.replace('.', File.separatorChar);
        buf.append(path);
        return buf.toString();
    }

    /**
     * Get the class file that we expect after XMLC has generated
     * a java file and called javac to compile it.
     */
    public File getClassFile() {
        File file = new File(baseOut() + ".class");
        return file;
    }
    public File getOptionFileForRecomp() {
       File file = new File(baseOut() + ".xmlc");
       return file;
    }

    /**
     * True if we can find the Java source file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewJavaFound() {
        boolean newFound = false;

        if (options.getKeep()) {
            newFound = options.getJavaClassSource().exists();
        }
        return newFound;
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    public void preCompile() throws XMLCException, IOException {
        options.complete();
        readOptionFiles();
        readParameters();

        // clear out any existing impl or standard classes.
        File implSource = null;
        StringBuffer implName = new StringBuffer();
        int index = -1;
        implName.append(options.getJavaClassSource());
        index = implName.toString().lastIndexOf(".");
        implName.insert(index, "Impl");
        implSource = new File(implName.toString());
        if (implSource.exists()) {
          implSource.delete();
        }
        if (options.getJavaClassSource().exists()) {
          options.getJavaClassSource().delete();
        }

        // reset names if needed
        if (options.getForRecomp()) {
          options.setJavaInterfaceSource(options.getJavaClassSource());
          options.setJavaClassSource(implSource);
        }
        resourceNode.preCompile();
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    public class Options extends XMLCOptions implements OptionSet {
        private File javaClassSource = null;
        private File javaInterfaceSource = null;
        private File sourceDoc = null;

        /**
         * Method declaration
         *
         *
         * @param f
         *
         * @see
         */
        public void setJavaClassSource(File f) {
            javaClassSource = f;
        }

        public void setJavaInterfaceSource(File f) {
           javaInterfaceSource = f;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        public File getJavaClassSource() {
            return javaClassSource;
        }

        public File getJavaInterfaceSource() {
           return javaInterfaceSource;
        }

        /**
         * Method declaration
         *
         *
         * @param f
         * @param real
         *
         * @see
         */
        public void setSourceDocument(File f, boolean real) {
            if (real) {
                sourceDoc = f;
                setSourceDocument(sourceDoc);
            }
        }

        /**
         * Method declaration
         *
         *
         * @param f
         *
         * @see
         */
        public void setSourceDocument(File f) {
            super.setSourceDocument(sourceDoc);
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        public File getSourceDocument() {
            return sourceDoc;
        }

    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.5 $
     */
    class LocalException extends Exception {
        private Exception baseException = null;

        /**
         * Constructor declaration
         *
         *
         * @param e
         *
         * @see
         */
        public LocalException(Exception e) {
            baseException = e;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        public String getMessage() {
            StringBuffer buf = new StringBuffer();
            String[]     parameters = getAllParameters(true);
            String[]     filenames = getAllOptionFilenames();

            buf.append(baseException.getMessage());
            buf.append("\n");
            if (parameters.length > 0) {
                buf.append("  Command line parameters: \n");
                for (int i = 0; i < parameters.length; i++) {
                    buf.append("     ");
                    buf.append(parameters[i]);
                    buf.append("\n");
                }
            }
            if (filenames.length > 0) {
                buf.append("  Option file");
                if (filenames.length > 1) {
                    buf.append("s");
                }
                buf.append(":\n");
                for (int i = 0; i < filenames.length; i++) {
                    buf.append("     ");
                    buf.append(filenames[i]);
                    buf.append("");
                }
            }
            return buf.toString().trim();
        }

    }
}
