/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: CodeImporter.java,v 1.3 2000/06/01 00:57:40 pmahar Exp $
 */
package org.enhydra.kelp.common.wizard;

// Kelp imports
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.node.Mapper;
import org.enhydra.kelp.common.node.OtterConfTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterResourceNode;
import org.enhydra.kelp.common.swing.ExtensionFilter;
import org.enhydra.kelp.common.wizard.conf.ConfBuildThread;

// Standard imports
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.3 $
 */
public class CodeImporter {
    private final static String KELP_PROPERTIES = "kelp.properties";
    private File                root = null;
    private File                source = null;
    private File                resource = null;
    private String[][]          packMap = new String[0][2];
    private String              pack = new String();
    private File                defaultSource = null;
    private String              defaultPackage = null;
    private ExtensionFilter     javaFilter = null;
    private ExtensionFilter     optionFilter = null;
    private ExtensionFilter     resourceFilter = null;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public CodeImporter() {
        javaFilter = new ExtensionFilter();
        javaFilter.setDirectoryValid(false);
        javaFilter.addExtension("java");
        optionFilter = new ExtensionFilter();
        optionFilter.setDirectoryValid(false);
        optionFilter.addExtension("xmlc");

        //
        resourceFilter = new ExtensionFilter();
        resourceFilter.setDirectoryValid(false);
        resourceFilter.addExtension("html");
        resourceFilter.addExtension("htm");
        resourceFilter.addExtension("wml");
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getRoot() {
        return root;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     *
     * @see
     */
    public void setRoot(File dir) throws ValidationException {
        if (ValidationUtil.isDirectory(dir)) {
            root = dir;
            defaultSource = null;
        } else {
            throw new ValidationException("Project root directory not found.");
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getSource() {
        return source;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     *
     * @see
     */
    public void setSource(File dir) throws ValidationException {
        if (ValidationUtil.isDirectory(dir)) {
            source = dir;
            defaultPackage = null;
        } else {
            throw new ValidationException("Source directory not found: \n"
                                          + dir.getAbsolutePath());
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getResource() {
        return resource;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     *
     * @see
     */
    public void setResource(File dir) throws ValidationException {
        if (ValidationUtil.isDirectory(dir)) {
            resource = dir;
        } else {
            throw new ValidationException("Resource directory not found: \n"
                                          + dir.getAbsolutePath());
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getDefaultSource() {
        File firstFound = null;

        if (defaultSource == null) {
            if (root != null) {
                defaultSource = root;
                firstFound = findFirst(javaFilter, root, 0);
            }
            if (firstFound != null) {
                String pack = readPackage(firstFound);
                int    depth = 0;

                if (ValidationUtil.isJavaPackage(pack)) {
                    if (pack.length() > 0) {
                        depth = charCount(pack, '.') + 1;
                    }
                }
                File foundParent = firstFound.getParentFile();

                while (depth > 0) {
                    foundParent = foundParent.getParentFile();
                    depth--;
                }
                defaultSource = foundParent;
            }
        }
        return defaultSource;
    }

    /**
     * Method declaration
     *
     *
     * @param in
     * @param lookFor
     *
     * @return
     *
     * @see
     */
    private int charCount(String in, char lookFor) {
        String search = new String(in);
        int    index = search.indexOf(lookFor);
        int    count = 0;

        while (index > 0) {
            count++;
            search = search.substring(index + 1);
            index = search.indexOf(lookFor);
        }
        return count;
    }

    /**
     * Method declaration
     *
     *
     * @param javaSource
     *
     * @return
     *
     * @see
     */
    private String readPackage(File javaSource) {
        BufferedReader in = null;
        String         line = null;
        String         pack = null;
        final String   key = "package ";

        try {
            in = new BufferedReader(new FileReader(javaSource));
            line = in.readLine();
            while (line != null) {
                line = line.trim();
                if (line.startsWith(key)) {
                    pack = line.substring(key.length(), line.indexOf(';'));
                    pack = pack.trim();
                    break;
                } else {
                    line = in.readLine();
                }
            }
            in.close();
        } catch (java.io.IOException e) {
            line = null;
        }
        return pack;
    }

    /**
     * Method declaration
     *
     *
     * @param makeFile
     *
     * @return
     *
     * @see
     */
    private String readXMLCParameters(File makeFile) {
        BufferedReader in = null;
        String         line = null;
        StringBuffer   param = new StringBuffer();
        final String   key = "XMLC_HTML_OPTS ";

        try {
            in = new BufferedReader(new FileReader(makeFile));
            line = in.readLine();
            while (line != null) {
                line = line.trim();
                if (line.startsWith(key)) {
                    int index = (line.indexOf('=') + 1);

                    while (line != null && line.endsWith("\\")) {
                        line = line.trim();
                        param.append(" ");
                        param.append(line.substring(index,
                                                    line.indexOf('\\')).trim());
                        index = 0;
                        line = in.readLine();
                    }
                    if (line != null) {
                        param.append(" ");
                        param.append(line.substring(index).trim());
                    }
                    break;
                } else {
                    line = in.readLine();
                }
            }
            in.close();
        } catch (java.io.IOException e) {
            line = null;
        }
        return param.toString().trim();
    }

    /**
     * Method declaration
     *
     *
     * @param file
     * @param token
     *
     * @return
     *
     * @see
     */
    private boolean fileContains(File file, String token) {
        BufferedReader in = null;
        String         line = null;
        boolean        found = false;

        try {
            in = new BufferedReader(new FileReader(file));
            line = in.readLine();
            while (line != null) {
                if (line.indexOf(token) > -1) {
                    found = true;
                    break;
                }
                line = in.readLine();
            }
            in.close();
        } catch (java.io.IOException e) {
            line = null;
        }
        return found;
    }

    /**
     * Method declaration
     *
     *
     * @param filter
     * @param dir
     *
     * @return
     *
     * @see
     */
    private File findFirst(FileFilter filter, File dir, int count) {
        File[] files = dir.listFiles(filter);
        File   found = null;
        File   candidate = null;

        if (count > 500) {

            // abort find if we've already searched through 500 directories
        } else {
            count++;
            if (files != null && files.length > 0) {
                found = files[0];
            }
            if (found == null) {
                files = dir.listFiles();
                if (files != null) {
                    for (int i = 0; i < files.length; i++) {
                        if (files[i].isDirectory()) {
                            if (files[i].getName().equalsIgnoreCase("classes")
                                    || files[i].getName().equalsIgnoreCase("output")) {

                                // ignore
                            } else {
                                candidate = findFirst(filter, files[i],
                                                      count);
                                if (found == null) {
                                    found = candidate;
                                } else if (candidate != null) {
                                    if (charCount(found.getAbsolutePath(), File.separatorChar)
                                            > charCount(candidate.getAbsolutePath(),
                                                        File.separatorChar)) {
                                        found = candidate;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return found;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getDefaultResource() {
        File   defaultResource = getResource();
        File   searchDir = null;
        String packPath = new String();
        File   firstFound = null;

        if (getPackage() == null) {
            packPath = getPackagePath();
        } else {
            packPath = getPackagePath(getDefaultPackage());
        }
        if (getSource() != null) {
            searchDir = getSource();
        } else if (getDefaultSource() != null) {
            searchDir = getDefaultSource();
        } else if (getRoot() != null) {
            searchDir = getRoot();
        }
        searchDir = new File(searchDir, packPath);
        firstFound = findFirst(resourceFilter, searchDir, 0);
        if (firstFound != null) {
            defaultResource = firstFound.getParentFile();
        }
        return defaultResource;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getPackage() {
        return pack;
    }

    /**
     * Method declaration
     *
     *
     * @param pac
     *
     * @exception ValidationException
     *
     * @see
     */
    public void setPackage(String pac) throws ValidationException {
        if (ValidationUtil.isJavaPackage(pac)) {
            pack = pac;
        } else {
            throw new ValidationException("Not a valid package name: " + pac);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getDefaultPackage() {
        String[] keys = {
            ".business", ".data", ".presentation"
        };
        File     firstFound = null;
        int      index = 0;

        if (defaultPackage == null) {
            if (getSource() != null) {
                firstFound = findFirst(javaFilter, getSource(), 0);
            }
            if (firstFound != null) {
                String inPack = readPackage(firstFound);

                for (int i = 0; i < keys.length; i++) {
                    index = inPack.indexOf(keys[i]);
                    if (index > 0) {
                        defaultPackage = inPack.substring(0, index);
                        break;
                    }
                    if (index < 0) {
                        index = inPack.indexOf(".");
                        if (index > 0) {
                            defaultPackage = inPack.substring(0, index);
                        } else {
                            defaultPackage = inPack;
                        }
                    }
                }
            }
        }
        return defaultPackage;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getPackageMap() {
        return packMap;
    }

    /**
     * Method declaration
     *
     *
     * @param m
     *
     * @see
     */
    public void setPackageMap(String[][] m) {
        packMap = m;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getDefaultPackageMap() {
        String[][] defaultMap = new String[1][2];
        File       firstFound = null;

        defaultMap[0][0] = getSource() + File.separator + getPackagePath()
                           + File.separator + "resources";
        defaultMap[0][1] = getPackage() + "." + "presentation";

        // That normally works, but lets check a different parallel
        // html to java structure.
        if (getResource() != null) {
            firstFound = findFirst(resourceFilter, getResource(), 0);
        }
        if (firstFound != null) {
            File   parent = firstFound.getParentFile().getParentFile();
            File[] dirList = parent.listFiles();
            File[] javaList = null;
            String baseName = firstFound.getName();

            baseName = baseName.substring(0, baseName.indexOf("."));
            for (int i = 0; i < dirList.length; i++) {
                if (dirList[i].isDirectory()) {
                    javaList = dirList[i].listFiles(javaFilter);
                    for (int j = 0; j < javaList.length; j++) {
                        if (javaList[j].getName().startsWith(baseName)) {
                            defaultMap[0][0] =
                                firstFound.getParentFile().getAbsolutePath();
                            defaultMap[0][1] = readPackage(javaList[j]);
                            break;
                        }
                    }
                }
            }
        }
        return defaultMap;
    }

    /**
     * Method declaration
     *
     *
     * @param project
     *
     * @see
     */
    public void importFilesFromRoot(OtterProject project) {
        importFiles(project, readFilesFromRoot());
    }

    /**
     * Method declaration
     *
     *
     * @param project
     * @param files
     *
     * @see
     */
    public void importFiles(OtterProject project, File[] files) {
        OtterConfTemplateNode[] confNodes = new OtterConfTemplateNode[0];

        preAdd(project);
        confNodes = addFiles(project, files);
        postAdd(project, confNodes);
    }

    /**
     * Method declaration
     *
     *
     * @param project
     *
     * @see
     */
    private void preAdd(OtterProject project) {
        File         firstFound = null;
        WebXMLFilter filter = new WebXMLFilter();

        setupProjectSourcePath(project);
        project.setProperty("Libraries", "Enhydra");
        project.setXMLCMake(project.isOpenBuild());
        project.setPackageMap(getPackageMap());
        firstFound = findFirst(filter, getSource(), 0);
        if (firstFound == null) {
            project.setResourceMap(true);
        } else if (getResource() != null) {
            project.setResourceMap(false);
            project.setResourcePath(getResource().getAbsolutePath());
        }
        addKelpProperties(project);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void postAdd(OtterProject project,
                         OtterConfTemplateNode[] nodes) {

        // Handler for Enhydra 3 style replacement set
        boolean outputToken = false;

        for (int i = 0; i < nodes.length; i++) {
            File template = new File(nodes[i].getFilePath());

            if (fileContains(template, "@OUTPUT@")) {
                outputToken = true;
                break;
            }
        }
        if (outputToken) {
            String[][] oldTable = project.getConfTable();
            String[][] newTable = new String[oldTable.length + 3][2];

            newTable[0][0] = "= ../classes";
            newTable[0][1] = "= @OUTPUT@/../classes";
            newTable[1][0] = "@OUTPUT@/../classes";
            newTable[1][1] = "@CLASSES@";
            newTable[2][0] = "@OUTPUT@";
            newTable[2][1] = project.getServletConfPath();
            for (int i = 0; i < oldTable.length; i++) {
                newTable[i + 3][0] = oldTable[i][0];
                newTable[i + 3][1] = oldTable[i][1];
            }
            project.setConfTable(newTable);
        }
        ConfBuildThread buildThread = null;

        buildThread = new ConfBuildThread();
        buildThread.setProject(project);
        buildThread.setNodes(nodes);
        buildThread.setPrintProgress(false);

        // buildThread.setStartMultiServerDirectory(new File(getAppDir()));
        buildThread.runInCurrentThread();

        // Handler for Enhydra 3 style startup
        boolean servlet = false;
        String  multi = null;
        String  path = new String();

        for (int i = 0; i < nodes.length; i++) {
            path = nodes[i].getConfFile().getAbsolutePath();
            if (path.endsWith("servlet.conf")) {
                servlet = true;
            } else if (path.endsWith("multiserver.conf")) {
                multi = path;
            }
        }
        if ((!servlet) && multi.length() > 0) {
            project.setConfStartupConf(multi);
            if (project.isConfStartupRun()) {
                project.configureRunClass();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private OtterConfTemplateNode[] addFiles(OtterProject project,
                                             File[] files) {
        OtterNodeFactory        nodeFactory = null;
        OtterResourceNode       node = null;
        OtterConfTemplateNode   conf = null;
        OtterConfTemplateNode[] confNodes = new OtterConfTemplateNode[0];
        Mapper                  mapper = new Mapper(project);
        Vector                  confVector = new Vector();

        nodeFactory = project.getNodeFactory();
        for (int i = 0; i < files.length; i++) {
            OtterNode folder = getFolder(project, nodeFactory, files[i]);
            String    path = files[i].getAbsolutePath();

            // Files to skip
            if (path.endsWith(".bak") || path.endsWith("~")
                    || path.endsWith(".jpr") || path.endsWith("tmp")) {

                // Exclude these files from import
                // Java source
            } else if (path.endsWith(".java")) {
                nodeFactory.createJavaFileNode(project, folder, path);

                // Images
            } else if (path.endsWith(".gif") || path.endsWith(".jpg")) {
                nodeFactory.createImageFileNode(project, folder, path);

                // HTML resources
            } else if (path.endsWith(".html")) {
                node = nodeFactory.createHTMLFileNode(project, folder, path);
                setupResourceNode(mapper, node);

                // Wireless WML
            } else if (path.endsWith(".wml")) {
                node = nodeFactory.createWMLFileNode(project, folder, path);
                setupResourceNode(mapper, node);

                // Configuration templates
            } else if (path.endsWith(".conf.in")) {
                conf = nodeFactory.createConfTemplateNode(project, folder,
                                                          path);
                conf.setSelected(true);
                confVector.addElement(conf);

                // Other files xmlc, xml, makefile, rul, conf, etc.
            } else {
                if (path.endsWith(CodeImporter.KELP_PROPERTIES)) {

                    // this file gets deleted in pre-add.
                } else {
                    nodeFactory.createTextFileNode(project, folder, path);
                }
            }
        }
        confNodes = (OtterConfTemplateNode[]) confVector.toArray(confNodes);
        return confNodes;
    }

    /**
     * Method declaration
     *
     *
     * @param project
     *
     * @see
     */
    private void addKelpProperties(OtterProject project) {
        StringBuffer    filePath = new StringBuffer();
        Properties      properties = new Properties();
        File            file;
        FileInputStream inStream;
        Enumeration     names;
        String          name;
        String          value;

        filePath.append(new File(project.getFilePath()).getParent());
        filePath.append(File.separator);
        filePath.append(KELP_PROPERTIES);
        file = new File(filePath.toString());
        if (file.exists()) {
            try {
                inStream = new FileInputStream(file);
                properties.load(inStream);
                names = properties.propertyNames();
                while (names.hasMoreElements()) {
                    name = (String) names.nextElement();
                    value = properties.getProperty(name);
                    project.setProperty(name, value);
                }
                inStream.close();
                properties.clear();
                file.delete();
            } catch (Exception e) {

                // non-fatal, report and continue
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param nodeFactory
     * @param javaFile
     *
     * @return
     *
     * @exception GeneratorException
     *
     * @see
     */
    private OtterNode getFolder(OtterProject project,
                                OtterNodeFactory nodeFactory,
                                File sourceFile) {

        // Source path + app package + folder + java file name =
        // java file absolute path.
        OtterNode folderNode = project;
        String    folderName = sourceFile.getParent();
        String    sourceRoot = getSource().getAbsolutePath() + File.separator;
        String    projectRoot = (new File(project.getFilePath()).getParent())
                                + File.separator;
        String    packagePath = getPackagePath() + File.separator;

        if (folderName.startsWith(sourceRoot)) {
            folderName = folderName.substring(sourceRoot.length());
            if (folderName.startsWith(packagePath)) {
                folderName = folderName.substring(packagePath.length());
                folderName = folderName.replace(File.separatorChar, '.');
            }
            folderNode = nodeFactory.createFolderNode(project, project,
                                                      folderName);
        } else if (folderName.startsWith(projectRoot)) {
            folderName = folderName.substring(projectRoot.length());
            folderName = folderName.replace('\\', '/');
            folderNode = nodeFactory.createFolderNode(project, project,
                                                      folderName);
        }
        return folderNode;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     *
     * @see
     */
    private String getPackagePath() {
        return getPackagePath(getPackage());
    }

    /**
     * Method declaration
     *
     *
     * @param pack
     *
     * @return
     *
     * @see
     */
    private String getPackagePath(String pack) {
        String path = new String();
        if (pack != null) {
          path = pack.replace('.', File.separatorChar);
        }
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @param mapper
     * @param node
     *
     * @exception GeneratorException
     *
     * @see
     */
    private void setupResourceNode(Mapper mapper, OtterResourceNode node) {
        String sourcePath = node.getFilePath();
        File   sourceFile = new File(sourcePath);

        if ((sourcePath != null)
                && (sourcePath.startsWith(getSource().getAbsolutePath()
                                          + File.separator + getPackagePath()
                                          + File.separator))) {
            File   file = null;
            File[] list = new File[0];
            String mappedPath = null;
            String sourceName = null;
            String javaName = null;

            // if .xmlc options file found in mapped path,
            // set xmlc file property
            mappedPath = mapper.getMappedSourcePath(node);
            file = new File(mappedPath);
            file = file.getParentFile();
            list = file.listFiles(optionFilter);
            if ((list != null) && (list.length > 0)) {
                node.setXMLCOptionFilename(list[0].getAbsolutePath());
            }

            // if XMLC_HTML_OPTS found in mapped path Makefile,
            // set as command line property.
            list = file.listFiles();
            for (int i = 0; i < list.length; i++) {
                if (list[i].getName().endsWith("Makefile")) {
                    String param = readXMLCParameters(list[i]);

                    if (param.length() > 0) {
                        node.setXMLCParameters(param);
                    }
                    break;
                }
            }
            list = file.listFiles(javaFilter);
            if (list != null) {
                for (int i = 0; i < list.length; i++) {
                    sourceName = sourceFile.getName().toLowerCase();
                    sourceName =
                        sourceName.substring(0, sourceName.lastIndexOf("."));
                    javaName = list[i].getName().toLowerCase();
                    if (javaName.startsWith(sourceName)) {
                        node.setSelected(true);
                        break;
                    }
                }
            }
            if (!node.isSelected()) {
                node.setStatic(true);
            }

            // if mapped package directory contains
            // an xmlc file than set it as the
            // String optionFilename = new String();
            // node.setXMLCOptionFilename(optionFilename);
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void setupProjectSourcePath(OtterProject project) {
        String[] path = project.getSourcePathArray();
        String[] newPath;
        File     current;
        String[] javaList;
        boolean  javaFound = false;

        for (int i = 0; i < path.length; i++) {
            current = new File(path[i]);
            javaList = current.list(new FilenameFilter() {

                /**
                 * Method declaration
                 *
                 *
                 * @param dir
                 * @param name
                 *
                 * @return
                 *
                 * @see
                 */
                public boolean accept(File dir, String name) {
                    return name.trim().toLowerCase().endsWith("java");
                }

            });
            if ((javaList != null) && (javaList.length > 0)) {
                javaFound = true;
            }
        }
        if (getSource().getAbsolutePath().trim().length() > 0) {
            if (javaFound) {
                newPath = new String[path.length + 1];
                newPath[0] = getSource().getAbsolutePath();
                for (int i = 1; i < (path.length + 1); i++) {
                    newPath[i] = path[i - 1];
                }
            } else {
                newPath = new String[1];
                newPath[0] = getSource().getAbsolutePath();
            }
            project.setSourcePathArray(newPath);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private File[] readFilesFromRoot() {
        File[] files = new File[0];

        if (getRoot() != null) {
            files = readFiles(getRoot());
        }
        return files;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     *
     * @see
     */
    private File[] readFiles(File dir) {
        File[] files = new File[0];
        Vector fileVector = new Vector();
        String classes = getRoot().getAbsolutePath() + File.separator
                         + "classes";
        String output = getRoot().getAbsolutePath() + File.separator
                        + "output";

        if (dir.isDirectory()) {
            File[] list = dir.listFiles();

            for (int i = 0; i < list.length; i++) {
                if (list[i].isDirectory()) {
                    if (list[i].getAbsolutePath().equalsIgnoreCase(classes)
                            || list[i].getAbsolutePath().equalsIgnoreCase(output)) {

                        // skip over classes and output folders.
                    } else {
                        File[] subList = readFiles(list[i]);

                        for (int j = 0; j < subList.length; j++) {
                            fileVector.addElement(subList[j]);
                        }
                    }
                } else if (list[i].isFile()) {
                    fileVector.addElement(list[i]);
                }
            }
        }
        if (fileVector.size() > 0) {
            files = (File[]) fileVector.toArray(files);
        }
        return files;
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.3 $
 */
class WebXMLFilter implements FileFilter {

    /**
     * Creates an extension file filter without an extensions set.
     */
    public WebXMLFilter() {}

    /**
     * Return true if this file should be shown in a file chooser.
     */
    public boolean accept(File f) {
        boolean acceptFile = false;

        if (f == null) {
            acceptFile = true;
        } else if (f.isDirectory()) {
            acceptFile = false;
        } else if (f.getName().equalsIgnoreCase("web.xml")) {
            acceptFile = true;
        }
        return acceptFile;
    }

}
