/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ImportMapPanel.java,v 1.1 2000/05/30 07:31:15 pmahar Exp $
 */
package org.enhydra.kelp.common.wizard.importer;

// Kelp imports
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.wizard.xmlc.MapPanel;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;

/**
 * Panel for entering the default options: project name, package
 * and destination.
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class ImportMapPanel extends ImporterPanel {
    transient private MapPanel      mapPanel;
    transient private JPanel        panelPack;
    transient private JTextField    textPack;

    //
    transient private JPanel        panelFiller;
    transient private GridBagLayout layoutMain;
    transient private GridBagLayout layoutPack;
    transient private TitledBorder  borderPack;

    /**
     * Create default option panel 1 for entering the
     * following default options: project name, package and
     * destination.
     */
    public ImportMapPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Read project name, package and destination values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptions() {
      String pack = null;
      String map[][] = new String[0][2];
      pack = getImporter().getPackage();
      if (pack != null) {
        textPack.setText(pack);
      }
      map = getImporter().getPackageMap();
      mapPanel.setMap(map);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void activated() {
        initPacks();
    }

    /**
     * Write project name, package and destination values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptions() throws ValidationException {
        String pack = new String();
        String[][] map = new String[0][2];

        if (getImporter() == null) {
            throw new ValidationException("Importer not set: cannot write options");
        }

        pack = textPack.getText().trim();
        getImporter().setPackage(pack);

        map = mapPanel.getMap();
        getImporter().setPackageMap(map);
    }

    /**
     * Validate project name, package and destination values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptions() throws ValidationException {
        String value = new String();

        // package
        value = textPack.getText().trim();
        if (!ValidationUtil.isJavaPackage(value)) {
            throw new ValidationException("Root package is not a valid Java package name.");
        }
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return "Package details";
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter package details. ");
        buf.append("All the application source files should be ");
        buf.append("defined at or below the root package. \n\n");
        buf.append("The package map is used to generate package names ");
        buf.append("for source files generated by XMLC.");
        return buf.toString();
    }

    // /
    // /

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        textPack = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        panelPack = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        mapPanel = (MapPanel) Beans.instantiate(getClass().getClassLoader(),
                                                MapPanel.class.getName());
        layoutPack =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        textPack.setText("PACKAGE");
        panelPack.setLayout(layoutPack);
        panelPack.add(textPack,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelPack,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(mapPanel,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        Border border = BorderFactory.createEtchedBorder();

        borderPack = BorderFactory.createTitledBorder(border);
        borderPack.setTitle("Project root package");
        panelPack.setBorder(borderPack);
        textPack.setText(new String());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initPacks() {
        String pack = null;
        String[][] map = new String[0][2];

        if (textPack.getText().trim().length() == 0) {
            pack = getImporter().getDefaultPackage();
            if (pack != null) {
                try {
                    getImporter().setPackage(pack);
                } catch (ValidationException e) {
                    e.printStackTrace();
                }
                textPack.setText(pack);
            }
        }
        if (mapPanel.getMap().length == 0) {
          map = getImporter().getDefaultPackageMap();
          mapPanel.setMap(map);
        }
    }

}
