/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: MapDialog.java,v 1.2 2000/05/30 07:21:03 pmahar Exp $
 */

package org.enhydra.kelp.common.wizard.xmlc;

import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.SwingTableSelectionEvent;
import org.enhydra.kelp.common.swing.SwingTableSelectionListener;
import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class MapDialog extends JDialog
       implements SwingTableSelectionListener {
    private boolean		standalone = false;
    private OtterProject	project = null;
    private LocalButtonListener buttonListener = null;
    private JPanel		panelMain;
    private MapPanel		mapPanel;
    private GridBagLayout       layoutMain;
    private JPanel		panelButton;
    private GridBagLayout	layoutButton;
    private JButton		buttonAdd;
    private JButton		buttonRemove;
    private JButton		buttonClose;
    private JPanel		jPanel1;
    private JPanel		panelFiller;
    private JButton             buttonEdit;

    /**
     * Constructor declaration
     *
     *
     * @param owner
     * @param title
     * @param modal
     *
     * @see
     */
    public MapDialog(Frame owner, String title, boolean modal) {
	super(owner, title, modal);

	construct();
    }

    /**
     * Constructor declaration
     *
     *
     * @param owner
     * @param title
     * @param modal
     *
     * @see
     */
    public MapDialog(Dialog owner, String title, boolean modal) {
	super(owner, title, modal);
	construct();
  }

    // overloading

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void show() {
	// setResizable(false); // crashes Win98

	Point cPoint;

	invalidate();
	doLayout();
	pack();

	cPoint = SwingUtil.getCenteringPoint(getSize());

	setLocation(cPoint);
	super.show();
    }

    // /
    // /

    public void onSwingTableSelection(SwingTableSelectionEvent event) {
      boolean en = (! event.isSelectionNull());
      buttonEdit.setEnabled(en);
      buttonRemove.setEnabled(en);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void clearAll() {
	layoutMain.removeLayoutComponent(mapPanel);
	panelMain.removeAll();

	layoutMain = null;
	panelMain = null;

	setTitle(null);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setProject(OtterProject p) {
	project = p;

	mapPanel.setMap(project.getPackageMap());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void closeDialog() {
	project.setPackageMap(mapPanel.getMap());
	setVisible(false);
	removeAll();
	dispose();

	if (standalone) {
	    System.exit(0);
	}
    }

    // /
    // /

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void construct() {
	try {
	    jbInit();
	    pmInit();
	    pack();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
	buttonListener = new LocalButtonListener();

 	buttonAdd.addActionListener(buttonListener);
	buttonEdit.addActionListener(buttonListener);
	buttonRemove.addActionListener(buttonListener);
	buttonClose.addActionListener(buttonListener);
        buttonEdit.setEnabled(false);
        buttonRemove.setEnabled(false);
        mapPanel.addSwingTableSelectionListener(this);

    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
	panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	mapPanel = (MapPanel) Beans.instantiate(getClass().getClassLoader(),
						MapPanel.class.getName());
	layoutMain =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	layoutButton =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
						JButton.class.getName());
	buttonRemove =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());
	buttonClose = (JButton) Beans.instantiate(getClass().getClassLoader(),
						  JButton.class.getName());
	jPanel1 = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					     JPanel.class.getName());
	panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());

	buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    panelMain.setLayout(layoutMain);
	panelButton.setLayout(layoutButton);
	buttonAdd.setText("Add...");
	buttonRemove.setText("Remove...");
	buttonClose.setText("Close");
	buttonEdit.setText("Edit...");
    panelMain.add(mapPanel,
		      new GridBagConstraints(0, 0, 1, 1, 0.7, 0.7,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(0, 0, 0, 0), 0, 0));
	panelMain.add(panelButton,
		      new GridBagConstraints(0, 1, 1, 1, 0.3, 0.3,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(0, 0, 0, 0), 20, 20));
	panelButton.add(buttonAdd, new GridBagConstraints(1, 0, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
	panelButton.add(buttonRemove, new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
	panelButton.add(buttonClose, new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
	panelButton.add(panelFiller, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    panelButton.add(buttonEdit, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
	this.getContentPane().add(panelMain);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void actionAdd() {
	MapEditorDialog d =
	    new MapEditorDialog(this, "Add a new directory to package mapping",
				 true);
        d.setProject(getProject());
	d.show();

	if (d.getOption() == MapEditorDialog.ACTION_OK) {
	    if (d.getFolder().trim().length() > 0) {
		mapPanel.addRow(d.getFolder(), d.getPackageName());
	    }
	}
    }

    private void actionEdit() {
	MapEditorDialog d =
	    new MapEditorDialog(this, "Edit directory to package mapping",
				 true);

        d.setFolder(mapPanel.getSelectedFolder());
        d.setPackageName(mapPanel.getSelectedPackageName());
        d.setProject(project);
	d.show();

	if (d.getOption() == MapEditorDialog.ACTION_OK) {
 	    if (d.getFolder().trim().length() > 0) {
              mapPanel.setSelectedFolder(d.getFolder());
              mapPanel.setSelectedPackageName(d.getPackageName());
//		mapPanel.addRow(d.getFolder(), d.getPackageName());
	    }
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void actionRemove() {
	if (mapPanel.getCurrentSelectionIndex() == MapPanel.NO_SELECTION) {

	    // does this ever happen?
	    // if so, would be good to have user alert here
	} else {
          int result = JOptionPane.showConfirmDialog(this,
             "Remove the current package mapping selection?",
             "Remove Mapping", JOptionPane.YES_NO_OPTION);
          if (result == JOptionPane.YES_OPTION) {
	     mapPanel.removeRow(mapPanel.getCurrentSelectionIndex());
	  }
	}
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 *
	 * @see
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonAdd) {
		actionAdd();
	    } else if (source == buttonEdit) {
		actionEdit();
	    } else if (source == buttonRemove) {
		actionRemove();
	    } else if (source == buttonClose) {
		closeDialog();
	    }
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

