/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: MapPanel.java,v 1.2 2000/05/30 07:21:03 pmahar Exp $
 */

package org.enhydra.kelp.common.wizard.xmlc;

import org.enhydra.kelp.common.swing.SwingTableSelectionEvent;
import org.enhydra.kelp.common.swing.SwingTableSelectionListener;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.beans.*;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class MapPanel extends JPanel implements ListSelectionListener {
    protected static final int NO_SELECTION = -1;
    private JTable	       table;
    private JScrollPane	       scrollTable;
    private BorderLayout       layoutMain;
    private MapTableModel      tableModel = new MapTableModel();
    private int		       currentSelectionIndex = NO_SELECTION;
    private Vector swingTableSelectionListeners = new Vector();
    private String[][] map = new String[0][2];

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public MapPanel() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }


  public synchronized void addSwingTableSelectionListener(SwingTableSelectionListener l) {
    if (! swingTableSelectionListeners.contains(l)) {
       swingTableSelectionListeners.addElement(l);
    }
  }

  public synchronized void removeSwingTableSelectionListener(SwingTableSelectionListener l) {
    if (swingTableSelectionListeners.contains(l)) {
      swingTableSelectionListeners.removeElement(l);
    }
  }

    /**
     * ListSelectionListener event
     */
    public void valueChanged(ListSelectionEvent e) {
	ListSelectionModel lsm = (ListSelectionModel) e.getSource();

	if (lsm.isSelectionEmpty()) {
	    currentSelectionIndex = NO_SELECTION;
	} else {
	    currentSelectionIndex = lsm.getMinSelectionIndex();
	}
        SwingTableSelectionListener listener = null;
        for (int i = 0 ; i < swingTableSelectionListeners.size() ; i++ ) {
          listener = (SwingTableSelectionListener) swingTableSelectionListeners.elementAt(i);
          listener.onSwingTableSelection(
             new SwingTableSelectionEvent(this, currentSelectionIndex));
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected int getCurrentSelectionIndex() {
	return currentSelectionIndex;
    }

    protected String getSelectedFolder() {
      String folder = null;
      if (currentSelectionIndex > -1) {
        folder = tableModel.getRow(currentSelectionIndex).getFolder();
      }
      return folder;
    }

    protected void setSelectedFolder(String folder) {
      if (currentSelectionIndex > -1) {
        tableModel.getRow(currentSelectionIndex).setFolder(folder);
        tableModel.fireTableDataChanged();
      }
    }

    protected String getSelectedPackageName() {
      String pack = null;
      if (currentSelectionIndex > -1) {
        pack = tableModel.getRow(currentSelectionIndex).getPackageName();
      }
      return pack;
    }

    protected void setSelectedPackageName(String pack) {
      if (currentSelectionIndex > -1) {
        tableModel.getRow(currentSelectionIndex).setPackageName(pack);
        tableModel.fireTableDataChanged();
      }
    }


    /**
     * Method declaration
     *
     *
     * @param f
     * @param p
     *
     * @see
     */
    protected void addRow(String f, String p) {
	tableModel.addRow(f, p);
    }

    /**
     * Method declaration
     *
     *
     * @param index
     *
     * @see
     */
    protected void removeRow(int index) {
	tableModel.removeRow(index);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
	layoutMain =
	    (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
					     BorderLayout.class.getName());
	table = (JTable) Beans.instantiate(getClass().getClassLoader(),
					   JTable.class.getName());
	scrollTable = new JScrollPane(table);

	table.setToolTipText(new String());
	table.sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
	table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
	table.setColumnSelectionAllowed(false);
    scrollTable.getViewport().add(table, BorderLayout.CENTER);

	ListSelectionModel rowSM = table.getSelectionModel();

	rowSM.addListSelectionListener(this);
	this.setPreferredSize(new Dimension(620, 200));
	this.setLayout(layoutMain);
	this.add(scrollTable, BorderLayout.CENTER);

    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getMap() {
        tableModel.readMap();
	return map;
    }

    /**
     * Method declaration
     *
     *
     * @param m
     *
     * @see
     */
    public void setMap(String[][] m) {
	map = m;

	tableModel.populateModel();
	table.setModel(tableModel);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
	tableModel = new MapTableModel();

	table.setModel(tableModel);
	table.setDefaultRenderer(String.class, new CellRendererWithToolTip());
	tableModel.addTableModelListener(table);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.getTableHeader().setUpdateTableInRealTime(false);
	table.getTableHeader().setReorderingAllowed(false);
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class MapTableModel extends AbstractTableModel {
	private Vector rowVector = new Vector();

	/**
	 * Constructor declaration
	 *
	 *
	 * @see
	 */
	public MapTableModel() {}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getRowCount() {
	    int count = 0;

	    if (rowVector != null) {
		count = rowVector.size();
	    }

	    return count;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getColumnCount() {
	    return 2;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public String getColumnName(int columnIndex) {
	    String name = new String();

	    switch (columnIndex) {

	    case 0:
		name = "Source Directory";

		break;

	    case 1:
		name = "Package";

		break;
	    }

	    return name;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Class getColumnClass(int columnIndex) {
	    Class  columnClass = null;
	    Object value = getValueAt(0, columnIndex);

	    if (value != null) {
		columnClass = value.getClass();
	    }

	    return columnClass;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
	    return false;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
	    Object value = null;

	    if (!isTableEmpty()) {
		MapRow  row = (MapRow) rowVector.elementAt(rowIndex);
		MapCell cell = new MapCell(columnIndex, row);

		value = cell.getValue();
	    }

	    return value;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param aValue
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @see
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	    if (!isTableEmpty()) {
		MapRow  row = (MapRow) rowVector.elementAt(rowIndex);
		MapCell cell = new MapCell(columnIndex, row);

		cell.setValue(aValue);
		fireTableCellUpdated(columnIndex, rowIndex);
	    }
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 *
	 * @return
	 *
	 * @see
	 */
	protected MapRow getRow(int rowIndex) {
	    MapRow row = null;

	    if (!isTableEmpty()) {
		if (rowVector.size() > rowIndex) {
		    row = (MapRow) rowVector.elementAt(rowIndex);
		}
	    }

	    return row;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected Object[] getWideValues() {
	    Object[] wideValues = null;
	    String   stringValue = " ";
	    String   current = " ";
	    int      rowCount = 0;

	    rowCount = getRowCount();
	    wideValues = new Object[2];
	    wideValues[0] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	    wideValues[1] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

	    for (int column = 1; column < wideValues.length; column++) {
		for (int row = 0; row < rowCount; row++) {
		    stringValue = (String) getValueAt(row, column);
		    stringValue = stringValue.trim();
		    current = (String) wideValues[column];

		    if (current == null) {
			wideValues[column] = stringValue;
		    } else {
			if (stringValue.trim().length() > current.length()) {
			    wideValues[column] = stringValue;
			}
		    }
		}
	    }

	    return wideValues;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void readMap() {
	    int	       rowCount = rowVector.size();
	    map = new String[rowCount][2];
	    MapRow     row = null;

	    for (int i = 0; i < rowCount; i++) {
		row = (MapRow) rowVector.elementAt(i);
		map[i][0] = row.getFolder();
		map[i][1] = row.getPackageName();
	    }
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void populateModel() {
	    if (map != null) {
		int rowCount = map.length;

		for (int i = 0; i < rowCount; i++) {
		    addRow(map[i][0], map[i][1]);
		}
	    }
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	private boolean isTableEmpty() {
	    boolean empty = true;

	    if (rowVector != null) {
		if (rowVector.size() > 0) {
		    empty = false;
		}
	    }

	    return empty;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	private void addRow(String f, String p) {
	    MapRow newRow = null;

	    newRow = new MapRow(f, p);

	    rowVector.addElement(newRow);

	    int size = rowVector.size();

	    fireTableDataChanged();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param index
	 *
	 * @see
	 */
	private void removeRow(int index) {
	    rowVector.removeElementAt(index);
	    fireTableDataChanged();
	}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class MapRow {
	private String fromFolder = new String();
	private String toPackage = new String();

	/**
	 * Constructor declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	public MapRow(String f, String p) {
	    fromFolder = f;
	    toPackage = p;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getFolder() {
	    return fromFolder;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 *
	 * @see
	 */
	public void setFolder(String f) {
	    fromFolder = f;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getPackageName() {
	    return toPackage;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param p
	 *
	 * @see
	 */
	public void setPackageName(String p) {
	    toPackage = p;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	private void saveToProject() {}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class MapCell {
	private MapRow row;
	private int    column;

	/**
	 * Constructor declaration
	 *
	 *
	 * @param c
	 * @param r
	 *
	 * @see
	 */
	public MapCell(int c, MapRow r) {
	    column = c;
	    row = r;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param value
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean setValue(Object value) {
	    boolean set = true;

	    switch (column) {

	    case 0:
		row.setFolder((String) value);

		break;

	    case 1:
		row.setPackageName((String) value);

		break;

	    default:
		set = false;

		break;
	    }

	    return set;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValue() {
	    Object value = null;

	    switch (column) {

	    case 0:
		value = row.getFolder();

		break;

	    case 1:
		value = row.getPackageName();

		break;
	    }

	    return value;
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected MapRow getRow() {
	    return row;
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

