/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCBuildTask.java,v 1.4 2000/06/02 08:32:34 pmahar Exp $
 */
package org.enhydra.kelp.jbuilder.build;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.common.wizard.xmlc.XMLCBuildThread;
import com.borland.primetime.build.*;
import com.borland.primetime.node.*;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.build.*;
import java.util.*;

/**
 * BuildTask implementation for calling XMLC.
 *
 */
public class XMLCBuildTask implements BuildTask, WriteListener {
    public static final Object SHARED_KEY = new Object();
    private BuildProcess       buildProcess;
    private Vector             nodeVector = new Vector();
    private Vector             writeListeners = new Vector();
    private XMLCBuildThread    buildThread;
    private int                messageCount = 0;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCBuildTask() {}

    /**
     * Generate the DOM class using XMLC.
     */
    public boolean build(BuildProcess buildProcess, Project project,
                         boolean rebuild) {
        this.buildProcess = buildProcess;
        OtterProject  otterProject = new PrimeProject((JBProject) project);
        int           nodeCount = nodeVector.size();
        int           compileCount = nodeCount;
        OtterXMLCNode current = null;

        // remove upToDate nodes from vector if not a rebuild.
        if (!rebuild) {
            for (int i = 0; i < nodeCount; i++) {
                current = (OtterXMLCNode) nodeVector.elementAt(i);
                if (upToDate(current)) {
                    nodeVector.setElementAt(new Object(), i);
                    compileCount--;
                }
            }
        }
        OtterXMLCNode[] nodes = new OtterXMLCNode[compileCount];

        compileCount = 0;
        for (int i = 0; i < nodeCount; i++) {
            if (nodeVector.elementAt(i) instanceof OtterXMLCNode) {
                if (compileCount < nodes.length) {
                    nodes[compileCount] =
                        (OtterXMLCNode) nodeVector.elementAt(i);
                    compileCount++;
                }
            }
        }
        WriteListener[] w = new WriteListener[1];

        w[0] = this;
        buildProcess.fireBuildStatus("Generating DOM classes with Enhydra XMLC",
                                     true);
        buildThread = new XMLCBuildThread(w, true);
        buildThread.setProject(otterProject);
        buildThread.setNodes(nodes);
        buildThread.setPrintProgress(false);
        buildThread.runInCurrentThread();
        for (int i = 0; i < nodes.length; i++) {
            nodes[i].replaceGeneratedSource(nodes[i].getOptions().getForRecomp());
        }
        return true;
    }

    /**
     * Build messages.
     *
     */
    public void onWrite(WriteEvent e) {
        if (buildProcess != null && messageCount < 200) {
            buildProcess.fireBuildMessage(e.getString(), e.getString());
            messageCount++;
            if (messageCount == 200) {
                String message =
                    "Reached maximum XMLC build message count of 200";

                buildProcess.fireBuildMessage(message, message);
            }
        }
    }

    /**
     * Collect list of nodes to build.
     *
     */
    public void addNode(OtterXMLCNode n) {
        nodeVector.addElement(n);
    }

    /**
     * Check if node up to date.
     *
     */
    private boolean upToDate(OtterXMLCNode xmlcNode) {
        boolean      built = false;
        HTMLFileNode htmlNode = (HTMLFileNode) xmlcNode.getNativeNode();

        if (htmlNode.isModified()) {
            try {
                htmlNode.save();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else if (!xmlcNode.getClassFile().exists()) {}
        else if (xmlcNode.getClassFile().lastModified()
                 > htmlNode.getUrl().getFileObject().lastModified()) {
            built = true;
        }
        return built;
    }

}
