/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: PrimeConfTemplateNode.java,v 1.1 2000/05/18 07:08:37 pmahar Exp $
 */

package org.enhydra.kelp.jbuilder.node;

import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterConfTemplateNode;
import org.enhydra.kelp.common.node.OtterSelectionNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.DuplicateNodeException;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PrimeConfTemplateNode extends PrimeNode
    implements OtterConfTemplateNode {

    private Exception exception;

    public PrimeConfTemplateNode(TextFileNode n) {
        super(n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @see
     */
    public PrimeConfTemplateNode(OtterProject otterProject, OtterNode otterNode,
		         String file) {
	Project  nativeProject = null;
	Node	 nativeNode = null;
	TextFileNode nativeTextFile = null;

	if (otterProject.getNativeProject() instanceof Project) {
	    nativeProject = (Project) otterProject.getNativeProject();
	}

	if (otterNode.getNativeNode() instanceof Node) {
	    nativeNode = (Node) otterNode.getNativeNode();
	}

	if (nativeProject != null && nativeNode != null) {
	    try {
		nativeTextFile = new TextFileNode(nativeProject, nativeNode,
					  new Url(new File(file)));
	    } catch (DuplicateNodeException e) {
                // Ignore: File already in project
	    }
	}

	setNativeNode(nativeTextFile);
    }


    public boolean isServer() {
      String in = getProperty(OtterConfTemplateNode.SERVER);
      return PropUtil.stringToBoolean(in, false);
    }

    public void setServer(boolean b) {
      setProperty(OtterConfTemplateNode.SERVER, PropUtil.booleanToString(b));
    }


  public boolean isSelected() {
    String in = getProperty(OtterSelectionNode.SELECTED);
    return PropUtil.stringToBoolean(in, false);
  }

  public void setSelected(boolean b) {
   setProperty(OtterSelectionNode.SELECTED, PropUtil.booleanToString(b));
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }

  public File getConfFile() {
    File out = null;
    String absolute = null;

    if (KelpVersion.getEnhydraVersion().startsWith("4") && isServer()) {
      absolute = getProject().getServletConfPath() +
          File.separator + "servlet.conf";
    } else {
      out = new File(getFilePath());
      absolute = getProject().getServletConfPath() +
          File.separator + out.getName();
      absolute = absolute.substring(0,absolute.length()-3);
    }

    out = new File(absolute);
    return out;
  }

}
