/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: PrimeProject.java,v 1.7 2000/06/02 08:32:34 pmahar Exp $
 */
package org.enhydra.kelp.jbuilder.node;

// Kelp imports
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterHTMLFileNode;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterWMLFileNode;
import org.enhydra.kelp.common.wizard.conf.ConfBuildThread;

// JB Foundation imports
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.paths.PathSet;
import com.borland.jbuilder.paths.PathSetManager;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.ImageFileNode;

// Standard imports
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PrimeProject extends OtterProject {
    private JBProject        nativeProject = null;
    private PrimeNodeFactory factory = null;

    /**
     * Constructor declaration
     *
     *
     * @param project
     *
     * @see
     */
    public PrimeProject(JBProject p) {
        nativeProject = p;
        factory = new PrimeNodeFactory();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode getFirstPolicy() {
        PolicyNodeReader reader = new PolicyNodeReader();

        return reader.getFirstNode(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterImageFileNode[] getAllImages() {
        ImageNodeReader reader = new ImageNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @param extensions
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode[] getAllText(String[] extensions) {
        TextNodeReader reader = new TextNodeReader(extensions);

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode[] getAllDeploymentDescs() {
        DeploymentNodeReader reader = new DeploymentNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterHTMLFileNode[] getAllHTMLFiles() {
        HTMLNodeReader reader = new HTMLNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterWMLFileNode[] getAllWMLFiles() {
        WMLNodeReader reader = new WMLNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeProject() {
        return nativeProject;
    }

    /**
     * Method declaration
     *
     *
     * @param o
     *
     * @see
     */
    public void setNativeProject(Object o) {
        nativeProject = (JBProject) o;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
        return getNativeProject();
    }

    /**
     * Method declaration
     *
     *
     * @param o
     *
     * @see
     */
    public void setNativeNode(Object o) {
        setNativeProject(o);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return this;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterNodeFactory getNodeFactory() {
        return factory;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isOpenBuild() {
        return true;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getName() {
        String name;

        name = nativeProject.getUrl().getName();
        return name;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getOutputDirectory() {
        String out = File.separator + "classes";

        if (nativeProject != null) {
            out = nativeProject.getPaths().getOutPath().getFile();
        }
        // Make match current OS.
        out.replace('\\', File.separatorChar);
        out.replace('/', File.separatorChar);
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getSourcePath() {
        String out = null;
        Url    urlSet[];

        if (nativeProject != null) {
            urlSet = nativeProject.getPaths().getSourcePath();
            for (int i = 0; i < urlSet.length; i++) {
                if (out != null) {
                    out += File.pathSeparator;
                }
                out += urlSet[i].getFile();
            }
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    public void setSourcePathArray(String[] pathAsArray) {
        Url urlSet[] = new Url[pathAsArray.length];

        for (int i = 0; i < pathAsArray.length; i++) {
            urlSet[i] = new Url(new File(pathAsArray[i]));
        }
        nativeProject.getPaths().setSourcePath(urlSet);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[] getSourcePathArray() {
        String[] pathAsArray = null;
        Url      urlSet[];
        File     file = null;

        if (nativeProject != null) {
            urlSet = nativeProject.getPaths().getSourcePath();
            pathAsArray = new String[urlSet.length];
            for (int i = 0; i < urlSet.length; i++) {

                // make paths match current OS;
                file = new File(urlSet[i].getFile());
                pathAsArray[i] = file.getAbsolutePath();
            }
        }
        return pathAsArray;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getClassPath() {
        PathSet pathSet;
        Url     urlSet[];
        String  path = null;
        String  libs = null;

        if (nativeProject != null) {
            libs = getProperty(JBProject.PROPERTY_LIBRARIES);
            pathSet = PathSetManager.getLibrary(libs);
            urlSet = pathSet.getFullClassPath();
            path = Url.getPath(urlSet);
        }
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    public String getProperty(String property) {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProperty(property);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    public void setProperty(String property, String value) {
        if (nativeProject != null) {
            nativeProject.setProperty(property, value);
        }
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param i
     *
     * @see
     */
    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilePath() {
        return nativeProject.getUrl().getFile();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
        return '/';
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void configureRunClass() {
        nativeProject.setProperty("runtime.0", "RunnableType",
                                  "com.borland.jbuilder.runtime.ApplicationRunner");
        nativeProject.setProperty("runtime.0", "application.class",
                                  KelpVersion.getMultiServerRunClass());
        if (KelpVersion.getEnhydraVersion().startsWith("3")) {
            nativeProject.setProperty("runtime.0", "application.parameters",
                                      getConfStartupConf());
        } else {
            StringBuffer value = new StringBuffer();
            StringBuffer policyPath = new StringBuffer();
            policyPath.append(getConfStartupConf());
            policyPath.append(File.separator);
            policyPath.append("java.policy");
            File policy = new File(policyPath.toString());

            if (!policy.exists()) {
                policy = ConfBuildThread.createPolicyFile(policyPath.toString());
            }
            value.append("org.enhydra.naming.service.NamingServiceManager ");
            value.append("org.enhydra.servlet.service.ServletServiceManager ");
            nativeProject.setProperty("runtime.0", "application.parameters",
                                       value.toString());
            value.setLength(0);
            value.append(" -Djava.security.policy=");
            value.append(policyPath);
            value.append(" -Denhydra.conf=");
            value.append(getConfStartupConf());
            nativeProject.setProperty("runtime.0",
                                      "application.vmparameters",
                                       value.toString());
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class ImageNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected ImageNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterImageFileNode[] getNodes(JBProject nativeProject) {
        OtterImageFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterImageFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterImageFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            ImageFileNode nativeImage = null;

            if (nativeNodes[i] instanceof ImageFileNode) {
                nativeImage = (ImageFileNode) nativeNodes[i];
                filename = nativeImage.getUrl().getFile();
                current = new File(filename);
                if (current.exists()) {
                    PrimeImageFileNode primeNode = null;

                    primeNode = new PrimeImageFileNode(nativeImage);
                    nodeVector.addElement(primeNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class DeploymentNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected DeploymentNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode[] getNodes(JBProject nativeProject) {
        OtterTextFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterTextFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterTextFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = new File(filename);
                if (current.exists()
                        && (current.getAbsolutePath().endsWith("web.xml")
                            || current.getAbsolutePath().endsWith("ejb-jar.xml"))) {
                    PrimeTextFileNode primeNode = null;

                    primeNode = new PrimeTextFileNode(nativeFile);
                    nodeVector.addElement(primeNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class HTMLNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected HTMLNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterHTMLFileNode[] getNodes(JBProject nativeProject) {
        OtterHTMLFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterHTMLFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterHTMLFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            HTMLFileNode nativeFile = null;

            if (nativeNodes[i] instanceof HTMLFileNode) {
                nativeFile = (HTMLFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = new File(filename);
                if (current.exists()) {
                    PrimeHTMLFileNode primeNode = null;

                    primeNode = new PrimeHTMLFileNode(nativeFile);
                    nodeVector.addElement(primeNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class WMLNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterWMLFileNode[] getNodes(JBProject nativeProject) {
        OtterWMLFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterWMLFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterWMLFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = new File(filename);
                if (current.exists()
                        && current.getAbsolutePath().endsWith(".wml")) {
                    PrimeWMLFileNode primeNode = null;

                    primeNode = new PrimeWMLFileNode(nativeFile);
                    nodeVector.addElement(primeNode);
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class TextNodeReader {
    private Vector   nodeVector = new Vector();
    private String[] extensions = new String[0];

    /**
     * Constructor declaration
     *
     *
     * @param exts
     *
     * @see
     */
    protected TextNodeReader(String[] exts) {
        extensions = exts;
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode[] getNodes(JBProject nativeProject) {
        OtterTextFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterTextFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterTextFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = new File(filename);
                for (int j = 0; j < extensions.length; j++) {
                    if (current.exists()
                            && current.getAbsolutePath().endsWith("."
                            + extensions[j])) {
                        PrimeTextFileNode primeNode = null;

                        primeNode = new PrimeTextFileNode(nativeFile);
                        nodeVector.addElement(primeNode);
                    }
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.7 $
 */
class PolicyNodeReader {
    private OtterTextFileNode node = null;

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode getFirstNode(JBProject nativeProject) {
        recursiveRead(nativeProject);
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Node container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            TextFileNode nativeFile = null;

            if (nativeNodes[i] instanceof TextFileNode) {
                nativeFile = (TextFileNode) nativeNodes[i];
                filename = nativeFile.getUrl().getFile();
                current = new File(filename);
                if (current.exists()
                        && current.getAbsolutePath().endsWith("java.policy")) {
                    node = new PrimeTextFileNode(nativeFile);
                    break;
                }
            }
            recursiveRead(nativeNodes[i]);
        }
    }

}
