
/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: WebAppWizard.java,v 1.1 2000/05/12 07:44:51 pmahar Exp $
 */
package org.enhydra.kelp.jdeveloper.wizard.webapp;

// Kelp imports
import org.enhydra.kelp.common.wizard.codegen.WebAppUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;

import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.jdeveloper.node.OracleProject;


// JDeveloper imports
import borland.jbuilder.addin.Addin;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;

// Standard imports
import java.awt.Image;
import java.io.File;
import java.util.Properties;
import javax.swing.JOptionPane;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class WebAppWizard implements Wizard, VersionedAddin {
    private WebAppUtil util = null;
    private String     imageFileName = new String();
    private Image      bigImage;

    /**
     * Method declaration
     *
     *
     * @param ide
     *
     * @see
     */
    public void startup(IdeEnvironment ide) {
        if (imageFileName.length() > 0) {

            // bigImage = imageImageLoader.loadFromResource(imageFileName,
            // ide.getIdeFrame(), getClass());
        }
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return "2.0";
    }

    /**
     * Method declaration
     *
     *
     * @param ide
     * @param context
     *
     * @see
     */
    public void invoke(IdeEnvironment ide, Context context) {
        Project nativeProject = null;
        int choice = WebAppUtil.CHOICE_CANCEL;

        if (context != null) {
            nativeProject = context.getProject();
            if (nativeProject == null
                    || nativeProject.getType().equals(Container.ROOT_TYPE)) {
                nativeProject = null;
                JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                              "Project not found. Please create a new project before "
                                              + "starting the new Enhydra Application wizard.");
            }
            if (nativeProject != null) {
                OtterProject otterProject = new OracleProject(nativeProject);

                try {
                    util = new WebAppUtil(otterProject);
                    fixSystemProperties(ide);
                    util.invokeWizard(ide.getIdeFrame());
                    if (util.getChoice() == WebAppUtil.CHOICE_FINISH) {
                    // Note steps should be added to replacement list!!!!
                        JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                                      "Enhydra Application files have been added to: \n"
                                                      + "     "
                                                      + nativeProject.getFilename()
                                                      + ".\n"
                                                      + "Refer to the Readme.html file for the steps \n "
                                                      + "required to compile and run your new XMLC \n"
                                                      + "application");
                    }
                } catch (CodeGenException e) {
                    JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                                  e.getMessage());
                }
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String toString() {
        return getName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getName() {
        return WebAppUtil.getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getMenuLabel() {
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Image getBigImage() {
        return IconSet.getLargeIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Image getSmallImage() {
        return IconSet.getSmallIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getInfo() {
        return Wizard.REQUIRES_PROJECT | Wizard.PRODUCES_FILE;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String[] getSteps() {
        String[] steps = new String[8];

        steps[0] =
            "Add <B>Enhydra</B> to the project's Java libraries list.<BR>";
        steps[1] = "Replace the current <i>Source root directories</i> "
                   + "setting with a single path to:<BR>" + "<B><code>"
                   + "appMaker.getAppDir()" + "</code></B><BR>";
        steps[2] = "Set <i>Output root directory</i> to: <BR>" + "<B><code>"
                   + "appMaker.getAppDir()" + "\\classes</code></B><BR>";
        steps[3] = "Select the File | Save All option commit all changes.";
        steps[4] =
            "From the menu select Wizards | XMLC Compiler and click on Compile "
            + "to compile the Welcome.html page into a DOM class.<BR>";
        steps[5] = "Make the project to compile the java files.";
        steps[6] =
            "Run the StartMultiServer program to launch the server.<BR>";
        steps[7] = "From your browser enter: http:\\localhost:9000<BR>";
        return steps;
    }

    // JDeveloper does not set system properties required
    // by LutrisAppTree. Set them here to workaround.

    /**
     * Method declaration
     *
     *
     * @param ide
     *
     * @see
     */
    private void fixSystemProperties(IdeEnvironment ide) {
        String home = System.getProperty("java.home");

        if (home == null || home.trim().length() == 0) {
            home = ide.getIdeRootDir();
            System.setProperty("java.home", home);
        }
        String path = System.getProperty("java.class.path");

        if (path == null || path.trim().length() == 0) {
            path = ide.getClassPath();
            System.setProperty("java.class.path", path);
        }
    }

}
